// Step 1: Clean raw data files to keep only the variables of interest. This step helps reduce the size of DTA files, making them more manageable. 

* 0) Preliminaries
*** Cleaning
	clear
	cls
	set more off
	set rmsg on																	// Keeps track of length of process
	
*** Install codes
   ssc install fs
   ssc install confirmdir
   ssc install chunky
   
*** Directories
global unzip "C:\Program Files\7-Zip\7z"				      					// Location of 7z executable program
global loc "CHANGE TO RELEVANT DRIVE"											// CHANGE DRIVE WHERE FORLDERS ARE LOCATED
global raw "$loc\financials\zip" 												// Location of zip files (SUGGESTION: COPY INTO THIS FOLDER ONLY THE ZIP FILES THAT YOU WANT TO CONVERT)
global txt "$loc\financial\txt"													// Output text files
global dta "$loc\financials\dta"												// Output data files 

global codes "$loc\processing_codes"											// Locataion to save log files
	
* 1) Largest financials file
clear
fs "$dta\intermediate\*.dta"
	return list
set obs 1 
	gen fn = ""
	gen id = 0
foreach f in `r(files)' {
	insobs 1
	replace id = id[_n-1] + 1 if _n > 1
	local name `f'
	replace fn = "`name'" if _n == _N
}
drop if id == 0
keep if strpos(fn,"industryglobalfinancialsandratiosusd") > 0
split fn, p(".") gen(fname)
	keep fname1 id
	rename fname1 fname
	gen string = substr(fname,-2,2)
	keep id string
		drop id
		gen id = _n
	 disp "List of files in processing loop"
save "$loc\financials\process.dta", replace
	qui sum id
	return list	
qui local NumericVar  exchangeratefromoriginalcurrency fixedassets intangiblefixedassets tangiblefixedassets otherfixedassets currentassets stock debtors othercurrentassets ///
          cashcashequivalent totalassets shareholdersfunds capital othershareholdersfunds noncurrentliabilities longtermdebt othernoncurrentliabilities provisions currentliabilities ///
		  loans creditors othercurrentliabilities totalsharehfundsliab workingcapital netcurrentassets enterprisevalue numberofemployees operatingrevenueturnover sales ///
		  costsofgoodssold grossprofit otheroperatingexpenses operatingplebit financialrevenue financialexpenses financialpl plbeforetax taxation plaftertax extrandotherrevenue ///
		  extrandotherexpenses extrandotherpl plforperiodnetincome exportrevenue materialcosts costsofemployees depreciationamortization interestpaid researchdevelopmentexpenses ///
		  addedvalue ebitda roeusingplbeforetax roceusingplbeforetax roausingplbeforetax roeusingnetincome roceusingnetincome roausingnetincome profitmargin grossmargin ///
		  ebitdamargin ebitmargin cashflowoperatingrevenue enterprisevalueebitdax marketcapcashflowfromoperationsx netassetsturnoverx interestcoverx stockturnoverx ///
		  collectionperioddays creditperioddays exportrevenueoperatingrevenue rdexpensesoperatingrevenue currentratiox shareholdersliquidityratiox solvencyratioassetbased ///
		  solvencyratioliabilitybased gearing profitperemployeeth operatingrevenueperemployeeth costsofemployeesoperatingrevenue averagecostofemployeeth ///
		  shareholdersfundsperemployeeth workingcapitalperemployeeth totalassetsperemployeeth	
forval i = `r(min)'(1)`r(max)' {
	use "$loc\financials\process.dta", clear
	noisily keep if id == `i'
	global st `=string[1]'	
	disp "$st"	
	use "$dta\intermediate\industryglobalfinancialsandratiosusd00$st.txt.dta", clear
	rename ïbvdidnumber bvdid
	gen ISO = substr(bvdid,1,2)
	destring `NumericVar', force replace
	save "$dta\intermediate\industryglobalfinancialsandratiosusd00$st.txt.dta", replace
}

erase "$loc\financials\process.dta"

* 2) Cash flow US industries USD
use "$dta\intermediate\cashflowusindustriesusd.dta", clear
	rename ïbvdidnumber bvdid
	gen ISO = substr(bvdid,1,2)
	tostring closingdate, replace
	gen Year = substr(closingdate1,4)
save "$dta\intermediate\cashflowusindustriesusd.dta", replace

* 3) Cash flow non US industries USD
use "$dta\intermediate\cashflownonusindustriesusd.dta", clear
	rename ïbvdidnumber bvdid
	gen ISO = substr(bvdid,1,2)
	tostring closingdate, replace
	gen Year = substr(closingdate1,4)
save "$dta\intermediate\cashflownonusindustriesusd.dta", replace

* 4) Banks global financial ratios USD
use "$dta\intermediate\banksglobalfinancialsratiosusd.dta", clear
	rename ïbvdidnumber bvdid
	gen ISO = substr(bvdid,1,2)
	tostring closingdate, replace
	gen Year = substr(closingdate1,4)
save "$dta\intermediate\banksglobalfinancialsratiosusd.dta", replace

* 5) DetailedFormatIndustriesUSD
use "$dta\intermediate\detailedformatindustriesusd.dta", clear
	rename ïbvdidnumber bvdid
	gen ISO= substr(bvdid,1,2)
	tostring closingdate, replace
	gen Year = substr(closingdate1,4)
save "$dta\intermediate\detailedformatindustriesusd.dta", replace

* 6) InsurancesGlobalFinancialsAndRatiosUSD
use "$dta\intermediate\insurancesglobalfinancialsandratiosusd.dta", clear
	rename ïbvdidnumber bvdid
	gen ISO = substr(bvdid,1,2)
	tostring closingdate, replace
	gen Year = substr(closingdate1,4)
save "$dta\intermediate\insurancesglobalfinancialsandratiosusd.dta", replace

* 7) KeyFinancialsUSD
use "$dta\intermediate\keyfinancialsusd.dta", clear
	rename ïbvdidnumber bvdid
	gen ISO = substr(bvdid,1,2)
	tostring closingdate, replace
	gen Year = substr(closingdate1,4)
save "$dta\intermediate\keyfinancialsusd.dta", replace
