// Step 3: Combine firm description files and save them in the appropriate country folder

*** Cleaning
	clear
	cls
	set more off
	set rmsg on																	// Keeps track of length of process
	
*** Install codes
   ssc install fs
   ssc install confirmdir
   ssc install chunky
   
*** Directories
global unzip "C:\Program Files\7-Zip\7z"				      					// Location of 7z executable program
global loc "CHANGE TO RELEVANT DRIVE"											// CHANGE DRIVE WHERE FORLDERS ARE LOCATED
global raw "$loc\firm_description\zip" 											// Location of zip files (SUGGESTION: COPY INTO THIS FOLDER ONLY THE ZIP FILES THAT YOU WANT TO CONVERT. THE GUIDE PROCESSES 12 OF THE 32 LOCATED ON THE ORBIS DRIVE IN FOLDER ORBIS DEC TEXT)
global txt "$loc\firm_description\txt"											// Output text files
global dta "$loc\firm_description\dta"											// Output data files 

global codes "$loc\processing_codes"											// Locataion to save log files

*** Create Country Folders (the first one is for the unstacked files)
use "$loc\countries_ISO.dta", clear
levelsof country, local(cntry)
foreach c of local cntry {
	confirmdir "$dta\intermediate_countries\\`c'"
	if r(confirmdir) == "170" {
		mkdir "$dta\intermediate_countries\\`c'"
	}
}

*** Create Country Folders (the second set is for stacked final files)
use "$loc\countries_ISO.dta", clear
levelsof country, local(cntry)
foreach c of local cntry {
	confirmdir "$dta\appended_countries\\`c'"
	if r(confirmdir) == "170" {
		mkdir "$dta\appended_countries\\`c'"
	}
}
	
* 1) Save intermediate files by country
disp "Generate list of individual files to process"
clear
fs "$dta\intermediate\*.dta"
	return list
set obs 1 
	gen fn = ""
	gen id = 0
foreach f in `r(files)' {
	insobs 1
	replace id = id[_n-1] + 1 if _n > 1
	local name `f'
	replace fn = "`name'" if _n == _N
}
drop if id == 0
split fn, p(".dta") gen(fname)
	keep fname1 id
	rename fname1 fname
	disp "List of files in processing loop"
	list fname
save "$loc\firm_description\files_all.dta", replace
	qui sum id
disp "Beginning to loop 1 through files: saving the data"	
forval i = `r(min)'(1)`r(max)'{
	use "$loc\firm_description\files_all.dta", clear
	keep if id == `i'
	global file `=fname[1]'
	
	use "$dta\\$file.dta", clear
	// Save individual country files
	levelsof ISO, local(cntry)
	foreach c of local cntry {
	preserve
		disp "Processing `c' portion of $file"
		keep if ISO == "`c'"
		drop ISO
		save "$dta\intermediate_countries\\`c'\\`c'_$file.dta", replace
	restore
	}		
	erase "$dta\\$file.dta"
}

erase "$loc\firm_description\files_all.dta"

*** NOTE: NOW WILL LOOP OVER EACH COUNTRY AND APPEND VARIOUS FILES AT THE COUNTRY LEVEL
foreach c of local cntry {
	* 2) Address files
	clear
	fs "$dta\intermediate_countries\\`c'\\*.dta"
		return list
	set obs 1 
		gen fn = ""
		gen id = 0
	foreach f in `r(files)' {
		insobs 1
		replace id = id[_n-1] + 1 if _n > 1
		local name `f'
		replace fn = "`name'" if _n == _N
	}
	drop if id == 0
	keep if strpos(fn,"address") > 0
	save "$loc\firm_description\process.dta", replace
	
	*** This sequence determines if the country has address files and generates a list of the address files that need to be processed
	count
	if `r(N)' != 0 {
		split fn, p(".") gen(fname)
			keep fname1 id
			rename fname1 fname
			gen string = substr(fname,-2,2)
			keep id string
				drop id
				gen id = _n
			 disp "List of files in processing loop"
		save "$loc\firm_description\process.dta", replace
			qui sum id
			return list
			count
	}
	*** If country has address files, then start with the first file in the sequence
	if `r(N)' != 0 {
		noisily keep if id == 1
		global st `=string[1]'
		disp "$st"
		use "$dta\intermediate_countries\\`c'\\`c'_all addresses00$st.txt.dta", clear
		preserve
			use "$loc\firm_description\process.dta", clear
			qui sum id
			return list
		restore
	}
	*** If country featured only in one file, finish the file
	if `r(N)' == 1 {
		drop if bvdid == ""
		duplicates drop		
		save "$dta\appended_countries\\`c'\\`c'_addresses_long.dta", replace
	}
	*** If country featured in multiple files, loop over all the files
	if `r(N)' > 1 {
		forval i = 2(1)`r(max)' {
			preserve
				use "$loc\firm_description\process.dta", clear
				noisily keep if id == `i'
				global st `=string[1]'	
				disp "$st"	
			restore
			append using "$dta\intermediate_countries\\`c'\\`c'_all addresses00$st.txt.dta"
		}
		drop if bvdid == ""
		duplicates drop
		save "$dta\appended_countries\\`c'\\`c'_addresses_long.dta", replace
	}

	*** NOTE: THIS STEP IS OPTIONAL, IT ERASES ALL OF THE INTERMDIATE FILES (WILL LEAVE THE INTERMEDIATE FOLDER EMPTY)
	use "$loc\firm_description\process.dta", clear
		qui sum id
		return list
	if `r(N)' == 1 {
		use "$loc\firm_description\process.dta", clear
		noisily keep if id == 1
		global st `=string[1]'	
		disp "$st"	
		erase "$dta\intermediate_countries\\`c'\\`c'_all addresses00$st.txt.dta"	
	}
	if `r(N)' > 1 {
		forval i = `r(min)'(1)`r(max)' {
			preserve
				use "$loc\firm_description\process.dta", clear
				noisily keep if id == `i'
				global st `=string[1]'	
				disp "$st"	
			restore
			erase "$dta\intermediate_countries\\`c'\\`c'_all addresses00$st.txt.dta"
		}
	}
	
	erase "$loc\firm_description\process.dta"

	* 3) Contact info (sequence of steps same as above)
	clear
	fs "$dta\intermediate_countries\\`c'\\*.dta"
		return list
	set obs 1 
		gen fn = ""
		gen id = 0
	foreach f in `r(files)' {
		insobs 1
		replace id = id[_n-1] + 1 if _n > 1
		local name `f'
		replace fn = "`name'" if _n == _N
	}
	drop if id == 0
	keep if strpos(fn,"contact") > 0
	save "$loc\firm_description\process.dta", replace
	
	count
	if `r(N)' != 0 {
		split fn, p(".") gen(fname)
			keep fname1 id
			rename fname1 fname
			gen string = substr(fname,-2,2)
			keep id string
				drop id
				gen id = _n
			 disp "List of files in processing loop"
		save "$loc\firm_description\process.dta", replace
			qui sum id
			return list
			count
	}
	if `r(N)' != 0 {
		noisily keep if id == 1
		global st `=string[1]'
		disp "$st"
		use "$dta\intermediate_countries\\`c'\\`c'_contact info00$st.txt.dta", clear
		preserve
			use "$loc\firm_description\process.dta", clear
			qui sum id
			return list
		restore
	}
	if `r(N)' == 1 {
		drop if bvdid == ""
		duplicates drop		
		save "$dta\appended_countries\\`c'\\`c'_contact_long.dta", replace
	}
	if `r(N)' > 1 {
		forval i = 2(1)`r(max)' {
			preserve
				use "$loc\firm_description\process.dta", clear
				noisily keep if id == `i'
				global st `=string[1]'	
				disp "$st"	
			restore
			append using "$dta\intermediate_countries\\`c'\\`c'_contact info00$st.txt.dta"
		}
		drop if bvdid == ""
		duplicates drop
		save "$dta\appended_countries\\`c'\\`c'_contact_long.dta", replace
	}

	*** NOTE: THIS STEP IS OPTIONAL, IT ERASES ALL OF THE INTERMDIATE FILES (WILL LEAVE THE INTERMEDIATE FOLDER EMPTY)
	use "$loc\firm_description\process.dta", clear
		qui sum id
		return list
	if `r(N)' == 1 {
		use "$loc\firm_description\process.dta", clear
		noisily keep if id == 1
		global st `=string[1]'	
		disp "$st"	
		erase "$dta\intermediate_countries\\`c'\\`c'_contact info00$st.txt.dta"	
	}
	if `r(N)' > 1 {
		forval i = `r(min)'(1)`r(max)' {
			preserve
				use "$loc\firm_description\process.dta", clear
				noisily keep if id == `i'
				global st `=string[1]'	
				disp "$st"	
			restore
			erase "$dta\intermediate_countries\\`c'\\`c'_contact info00$st.txt.dta"
		}
	}
	
	erase "$loc\firm_description\process.dta"

	* 3) Identifiers
	clear
	fs "$dta\intermediate_countries\\`c'\\*.dta"
		return list
	set obs 1 
		gen fn = ""
		gen id = 0
	foreach f in `r(files)' {
		insobs 1
		replace id = id[_n-1] + 1 if _n > 1
		local name `f'
		replace fn = "`name'" if _n == _N
	}
	drop if id == 0
	keep if strpos(fn,"identifiers") > 0
	save "$loc\firm_description\process.dta", replace
	
	count
	if `r(N)' != 0 {
		split fn, p(".") gen(fname)
			keep fname1 id
			rename fname1 fname
			gen string = substr(fname,-2,2)
			keep id string
				drop id
				gen id = _n
			 disp "List of files in processing loop"
		save "$loc\firm_description\process.dta", replace
			qui sum id
			return list
			count
	}
	if `r(N)' != 0 {
		noisily keep if id == 1
		global st `=string[1]'
		disp "$st"
		use "$dta\intermediate_countries\\`c'\\`c'_identifiers00$st.txt.dta", clear
		preserve
			use "$loc\firm_description\process.dta", clear
			qui sum id
			return list
		restore
	}
	if `r(N)' == 1 {
		drop if bvdid == ""
		duplicates drop		
		save "$dta\appended_countries\\`c'\\`c'_identifiers_long.dta", replace
	}
	if `r(N)' > 1 {
		forval i = 2(1)`r(max)' {
			preserve
				use "$loc\firm_description\process.dta", clear
				noisily keep if id == `i'
				global st `=string[1]'	
				disp "$st"	
			restore
			append using "$dta\intermediate_countries\\`c'\\`c'_identifiers00$st.txt.dta"
		}
		drop if bvdid == ""
		duplicates drop
		save "$dta\appended_countries\\`c'\\`c'_identifiers_long.dta", replace
	}
	
	*** NOTE: THIS STEP IS OPTIONAL, IT ERASES ALL OF THE INTERMDIATE FILES (WILL LEAVE THE INTERMEDIATE FOLDER EMPTY)
	use "$loc\firm_description\process.dta", clear
		qui sum id
		return list
	if `r(N)' == 1 {
		use "$loc\firm_description\process.dta", clear
		noisily keep if id == 1
		global st `=string[1]'	
		disp "$st"	
		erase "$dta\intermediate_countries\\`c'\\`c'_identifiers00$st.txt.dta"	
	}
	if `r(N)' > 1 {
		forval i = `r(min)'(1)`r(max)' {
			preserve
				use "$loc\firm_description\process.dta", clear
				noisily keep if id == `i'
				global st `=string[1]'	
				disp "$st"	
			restore
			erase "$dta\intermediate_countries\\`c'\\`c'_identifiers00$st.txt.dta"
		}
	}

	erase "$loc\firm_description\process.dta"
	
	* 4) Industry Classifications
		// Industry
	*** Append
	clear
	fs "$dta\intermediate_countries\\`c'\\*.dta"
		return list
	set obs 1 
		gen fn = ""
		gen id = 0
	foreach f in `r(files)' {
		insobs 1
		replace id = id[_n-1] + 1 if _n > 1
		local name `f'
		replace fn = "`name'" if _n == _N
	}
	drop if id == 0
	keep if strpos(fn,"industry") > 0
	save "$loc\firm_description\process.dta", replace
	count
	
	if `r(N)' != 0 {
		split fn, p(".") gen(fname)
			keep fname1 id
			rename fname1 fname
			gen string = substr(fname,-2,2)
			keep id string
				drop id
				gen id = _n
			 disp "List of files in processing loop"
		save "$loc\firm_description\process.dta", replace
			qui sum id
			return list
			count
	}
	
	if `r(N)' != 0 {
		noisily keep if id == 1
		global st `=string[1]'
		disp "$st"
		use "$dta\intermediate_countries\\`c'\\`c'_industry classifications00$st.txt.dta", clear
		drop if bvdid == ""
		duplicates drop		
		preserve
			use "$loc\firm_description\process.dta", clear
			qui sum id
			return list
		restore
	}
	if `r(N)' == 1 {
		drop if bvdid == ""
		duplicates drop		
		save "$dta\appended_countries\\`c'\\`c'_industries_long.dta", replace
	}
	if `r(N)' > 1 {
		forval i = 2(1)`r(max)' {
			preserve
				use "$loc\firm_description\process.dta", clear
				noisily keep if id == `i'
				global st `=string[1]'	
				disp "$st"	
			restore
			append using "$dta\intermediate_countries\\`c'\\`c'_industry classifications00$st.txt.dta"
		}
		drop if bvdid == ""
		duplicates drop
		save "$dta\appended_countries\\`c'\\`c'_industries_long.dta", replace
	}

	*** NOTE: THIS STEP IS OPTIONAL, IT ERASES ALL OF THE INTERMDIATE FILES (WILL LEAVE THE INTERMEDIATE FOLDER EMPTY)
	use "$loc\firm_description\process.dta", clear
		qui sum id
		return list
	if `r(N)' == 1 {
		use "$loc\firm_description\process.dta", clear
		noisily keep if id == 1
		global st `=string[1]'	
		disp "$st"	
		erase "$dta\intermediate_countries\\`c'\\`c'_industry classifications00$st.txt.dta"	
	}
	if `r(N)' > 1 {
		forval i = `r(min)'(1)`r(max)' {
			preserve
				use "$loc\firm_description\process.dta", clear
				noisily keep if id == `i'
				global st `=string[1]'	
				disp "$st"	
			restore
			erase "$dta\intermediate_countries\\`c'\\`c'_industry classifications00$st.txt.dta"
		}
	}
	
	erase "$loc\firm_description\process.dta"
	
	* 5) Legal information
	clear
	fs "$dta\intermediate_countries\\`c'\\*.dta"
		return list
	set obs 1 
		gen fn = ""
		gen id = 0
	foreach f in `r(files)' {
		insobs 1
		replace id = id[_n-1] + 1 if _n > 1
		local name `f'
		replace fn = "`name'" if _n == _N
	}
	drop if id == 0
	keep if strpos(fn,"legal") > 0
	save "$loc\firm_description\process.dta", replace
	
	count
	if `r(N)' != 0 {
		split fn, p(".") gen(fname)
			keep fname1 id
			rename fname1 fname
			gen string = substr(fname,-2,2)
			keep id string
				drop id
				gen id = _n
			 disp "List of files in processing loop"
		save "$loc\firm_description\process.dta", replace
			qui sum id
			return list
			count
	}
	if `r(N)' != 0 {
		noisily keep if id == 1
		global st `=string[1]'
		disp "$st"
		use "$dta\intermediate_countries\\`c'\\`c'_legal info00$st.txt.dta", clear
		preserve
			use "$loc\firm_description\process.dta", clear
			qui sum id
			return list
		restore
	}
	if `r(N)' == 1 {
		drop if bvdid == ""
		duplicates drop	
		save "$dta\appended_countries\\`c'\\`c'_legalinfo_long.dta", replace
	}
	if `r(N)' > 1 {
		forval i = 2(1)`r(max)' {
			preserve
				use "$loc\firm_description\process.dta", clear
				noisily keep if id == `i'
				global st `=string[1]'	
				disp "$st"	
			restore
			append using "$dta\intermediate_countries\\`c'\\`c'_legal info00$st.txt.dta"
		}
		drop if bvdid == ""
		duplicates drop
		save "$dta\appended_countries\\`c'\\`c'_legalinfo_long.dta", replace
	}

	*** NOTE: THIS STEP IS OPTIONAL, IT ERASES ALL OF THE INTERMDIATE FILES (WILL LEAVE THE INTERMEDIATE FOLDER EMPTY)
	use "$loc\firm_description\process.dta", clear
		qui sum id
		return list
	if `r(N)' == 1 {
		use "$loc\firm_description\process.dta", clear
		noisily keep if id == 1
		global st `=string[1]'	
		disp "$st"	
		erase "$dta\intermediate_countries\\`c'\\`c'_legal info00$st.txt.dta"	
	}
	if `r(N)' > 1 {
		forval i = `r(min)'(1)`r(max)' {
			preserve
				use "$loc\firm_description\process.dta", clear
				noisily keep if id == `i'
				global st `=string[1]'	
				disp "$st"	
			restore
			erase "$dta\intermediate_countries\\`c'\\`c'_legal info00$st.txt.dta"
		}
	}
	
	erase "$loc\firm_description\process.dta"
}

